from typing import Any

from aqueduct.utils.type_inference import infer_artifact_type
from aqueduct_executor.operators.function_executor.execute import execute_function_spec
from aqueduct_executor.operators.function_executor.spec import FunctionSpec
from aqueduct_executor.operators.spark.utils import read_artifacts_spark, write_artifact_spark
from aqueduct_executor.operators.utils.enums import ArtifactType
from pyspark.sql import SparkSession, dataframe


def infer_artifact_type_spark(value: Any) -> Any:
    if isinstance(value, dataframe.DataFrame):
        return ArtifactType.TABLE
    else:
        return infer_artifact_type(value)


def run(spec: FunctionSpec, spark_session_obj: SparkSession) -> None:
    """
    Executes a function operator.
    """
    return execute_function_spec(
        spec=spec,
        read_artifacts_func=read_artifacts_spark,
        write_artifact_func=write_artifact_spark,
        infer_type_func=infer_artifact_type_spark,
        spark_session_obj=spark_session_obj,
    )



import argparse
import base64
import sys

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--spec", required=True)
    parser.add_argument("-v", "--version-tag", default="")
    args = parser.parse_args()
    if args.version_tag:
        import subprocess

        install_process = subprocess.run(
            [
                sys.executable,
                "-m",
                "pip",
                "install",
                "--index-url",
                "https://test.pypi.org/simple/",
                "--extra-index-url",  # allows dependencies from pypi
                "https://pypi.org/simple",
                f"aqueduct-ml=={args.version_tag}",
            ]
        )
        print(install_process.stderr)
        print(install_process.stdout)
        install_process.check_returncode()

    from aqueduct_executor.operators.function_executor import execute
    from aqueduct_executor.operators.function_executor.spec import parse_spec
    from aqueduct_executor.operators.utils.utils import time_it

    spec_json = base64.b64decode(args.spec)
    spec = parse_spec(spec_json)

    time_it(job_name=spec.name, job_type=spec.type.value, step="Running Operator (including IO)")(
        execute.run
    )(spec)
